# #########################################################################################################################################
# File: tmPacketsTrapFPA.tcl
#
# Monitoring script that shall be launched inside VIS switch on script. It tries to catch 5,3 from VIS and implement
# CDMU FDIR that switch on and off RPSU according to requests from instrument.
# It closes the monitoring script when both CDPU are switched off.
#
# History:
# 19/03/2019 - (E.Castaldini) First Issue
#
# 03/04/2020 - (M. Whillock) Changes for FPS thermal tests.
#                            Changed TM packet and TM parameter values.
#                            Changed format of ascii output file to csv.
#                            Added common file variable.
#                            Both trap procedures write to the same file.
#
# #########################################################################################################################################

# Spids: 310111010 vis asw tc accept ok
#        310112010 vis asw tc accept nok
#        310133002 st slow housekeeping
#        310134013 roe hk lim rep
#        310151035 VIS SHUTTER OPEN

# TM:    VS1T2084 VIS_ROE_FPGA_TEMP_B_01 st slow
#        VP1T2509 VIS_RPSU_VAN_N_V01     st slow
#        VP3T2510 VIS_RPSU_VAN_P_I03     st superfast
#        VP7T2511 VIS_RPSU_VAN_N_I07     st superfast
#        VP1T2506 VIS_RPSU_28V_PRV01     rt slow
#        VS1T2020 VIS_ROE_CCD1_TTO01     rt slow
#        VP2T2503 VIS_RPSU_VCCD_I_02     rt fast
#        VP8T2502 VIS_RPSU_VCCD_V_08     rt fast
#        VMGT3068 VIS_PMCU_V_THCU2       rt superfast
#        VP9T2512 VIS_RPSU_3V3_DIG09     rt superfast

# Events: ?

# TC: ?

#
# DARTS dump format:
# TM,trx,id,spid,name,apid,ssc,type,stype,pi1,vc,obt,obtOk,crcFlag,crcOk,alarm,warning
# TMPAR,name,rawValue,engValue,monState,sample,packetRef
# EV,logTime,entryNo,severity,source,workstation,message
#

namespace eval tmPacketsTrap {
    set ::tmPacketsTrap::listOfSpid {310111010 310112010 310133002 310134013 310151035}
    set ::tmPacketsTrap::listOfTM {VS1T2084 VP1T2509 VP3T2510 VP7T2511 VP1T2506 VS1T2020 VP2T2503 VP8T2502 VMGT3068 VP9T2512}

    set ::tmPacketsTrap::outputFile "C:\\TEMP\\FPA_thermal.txt"

    # #######################################################################################################################################
    # Subroutine: ::tmPacketsTrap::writeLineToFile
    # Write a line of text to file. Appending at the end. If the file doesn't exist it creates it.
    #
    # Parameters:
    #     textData    - The text to be written to file
    #     filename    - The name of the file to be written on in append (and created if needed). Use \\ in tcl for the folder char \
	#     newLineNeeded - (default: YES) If YES (uppercase needed) it go to new line after the line of text
    #
    # Return:
    # Nothing
    # #######################################################################################################################################
    proc ::tmPacketsTrap::writeLineToFile { textData filename {newLineNeeded YES} } {
	# open the filename for writing
	set fileId [open $filename "a"]
	# send the data to the file -
	#  omitting '-nonewline' will result in an extra newline
	# at the end of the file
	if { $newLineNeeded == "YES" } {
	    puts $fileId $textData
	} else {
	    puts -nonewline $fileId $textData
	}
	# close the file, ensuring the data is written out before you continue
	#  with processing.
	close $fileId
	
    }
    # End of ::L2UTIL::writeLineToFile
    
    # Placeholders in output string until we can extract the correct values
    proc ::tmPacketsTrap::trapPacketProcedure {name index operation} {
	set spid [getspid $::tmPacketsTrap::thePacket]
	set identifier [string range $name [expr [string first _ $name] + 1] end]
	upvar $name packet
	set packetRawData [getrawdata $packet]
	set operationId [string range $packetRawData 16 19]
	
	set obt [getobt $packet]
	syslog "Caught TM - $obt-$spid"
	
	#::tmPacketsTrap::writeLineToFile "$obt\t$packetRawData" "C:\\TEMP\\TMFILE.txt"
	::tmPacketsTrap::writeLineToFile "$obt,id,$spid,name,apid,ssc,type,stype,pi1,vc,$obt,obtOk,crcFlag,crcOk,alarm,warning" $::tmPacketsTrap::outputFile
    }
    
    # Placeholders in output string until we can extract the correct values
    proc ::tmPacketsTrap::trapTMParamProcedure { a b c} {
	# 'a' contains the variable name (in this case var2, but you could be multiple variables).
	# 'b' contains the empty string or an array index if 'a' refers to an array variable.
	# 'c' contains one of { read write unset } depending on what triggered the trace.
	if {$c == "write"} {	
	    set x [string range $a 17 24]
	    set newLine ""
	    set thisTM [fetch $x]
	    # check if TM has come together with this packet
	    set tmObt [getobt $thisTM]
	    set tmRaw [getrawvalue $thisTM]
	    set tmEng [getengvalue $thisTM]
	    set tmOol [getoolstate $thisTM]
	    #set newLine "$tmObt,$tmRaw,$tmEng,$tmOol"
	    # To be the same as DARTS dumps also need sample and packetRef after tmOol - how to extract these?
	    set newLine "TMPAR,$x,$tmRaw,$tmEng,$tmOol,sample,packetRef"
	    #::tmPacketsTrap::writeLineToFile $newLine "C:\\TEMP\\$x.txt"
	    ::tmPacketsTrap::writeLineToFile $newLine $::tmPacketsTrap::outputFile
	}
    }

    proc ::tmPacketsTrap::mainLoop {} {
	# packets sid definition
	
	# subscribe packets
	foreach thisPacket $::tmPacketsTrap::listOfSpid {
	    syslog "Subscribe packet $thisPacket"
	    subscribepacket $thisPacket referby ::tmPacketsTrap::thePacket
	    syslog "Trace change in packet $thisPacket"
	    trace add variable ::tmPacketsTrap::thePacket write {::tmPacketsTrap::trapPacketProcedure}
	}
	
	# subscribe tm parameters
	foreach x $::tmPacketsTrap::listOfTM {
	    subscribeparam $x referby ::tmPacketsTrap::$x
	    trace add variable ::tmPacketsTrap::$x write {::tmPacketsTrap::trapTMParamProcedure}	
	}
	
	set exitLoop 0
	while {$exitLoop == 0} {
	    # check for packets arriving
	    waitfor -timeout 5.000 {::tmPacketsTrap::thePacket}
	    # Do we need another check for TM params?
	    # check exit condition and in case exit			
	}
	
	# unsubscribe packets
	foreach thisPacket $::tmPacketsTrap::listOfSpid {
	    syslog "Trace delete in packet $thisPacket"
	    trace delete variable ::tmPacketsTrap::thePacket
	    syslog "Unsubscribe packet $thisPacket"
	    unsubscribepacket $thisPacket 	    
	}
	
    }
    
    ::tmPacketsTrap::mainLoop
    
}
# End of namespace
